var _loadScript=function(e,t,r){var n=document.createElement("script");n.type="text/javascript",n.readyState?n.onreadystatechange=function(){"loaded"!=n.readyState&&"complete"!=n.readyState||(n.onreadystatechange=null,r())}:n.onload=function(){r()};var a=["type","src","htmlFor","event","charset","async","defer","crossOrigin","text","onerror"];if("object"==typeof t&&!jQuery.isEmptyObject(t))for(var o in t)t.hasOwnProperty(o)&&jQuery.inArray(o,a)&&(n[o]=t[o]);n.src=e,document.getElementsByTagName(t.lazyLoad?"body":"head")[0].appendChild(n)};jQuery.loadScript=function(e,t,r){2===arguments.length&&"function"==typeof arguments[1]&&(r=arguments[1],t={}),t=t||{};var n=jQuery.Deferred();return"function"==typeof r&&n.done((function(){r()})),_loadScript(e,t,(function(){n.resolve()})),n.promise()};